# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: aiExchangeMessages.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='aiExchangeMessages.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\220\001\000'),
  serialized_pb=_b('\n\x18\x61iExchangeMessages.proto\"\"\n\x0b\x44\x61taRequest\x12\x13\n\x0brequest_ids\x18\x01 \x03(\t\"\xf5\x07\n\x0c\x44\x61taResponse\x12%\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x17.DataResponse.DataEntry\x1a\xfc\x06\n\x04\x44\x61ta\x12/\n\x08position\x18\x01 \x01(\x0b\x32\x1b.DataResponse.Data.PositionH\x00\x12)\n\x05speed\x18\x02 \x01(\x0b\x32\x18.DataResponse.Data.SpeedH\x00\x12\x31\n\x05\x61ngle\x18\x03 \x01(\x0b\x32 .DataResponse.Data.SteeringAngleH\x00\x12)\n\x05lidar\x18\x04 \x01(\x0b\x32\x18.DataResponse.Data.LidarH\x00\x12+\n\x06\x63\x61mera\x18\x05 \x01(\x0b\x32\x19.DataResponse.Data.CameraH\x00\x12+\n\x06\x64\x61mage\x18\x06 \x01(\x0b\x32\x19.DataResponse.Data.DamageH\x00\x12\x45\n\x14lane_center_distance\x18\x07 \x01(\x0b\x32%.DataResponse.Data.LaneCenterDistanceH\x00\x12>\n\x11\x63\x61r_to_lane_angle\x18\x08 \x01(\x0b\x32!.DataResponse.Data.CarToLaneAngleH\x00\x12\x36\n\x0c\x62ounding_box\x18\t \x01(\x0b\x32\x1e.DataResponse.Data.BoundingBoxH\x00\x12)\n\x05\x65rror\x18\n \x01(\x0b\x32\x18.DataResponse.Data.ErrorH\x00\x1a \n\x08Position\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\x1a\x16\n\x05Speed\x12\r\n\x05speed\x18\x01 \x01(\x01\x1a\x1e\n\rSteeringAngle\x12\r\n\x05\x61ngle\x18\x01 \x01(\x01\x1a\x17\n\x05Lidar\x12\x0e\n\x06points\x18\x01 \x03(\x01\x1a\x39\n\x06\x43\x61mera\x12\r\n\x05\x63olor\x18\x01 \x01(\x0c\x12\x11\n\tannotated\x18\x02 \x01(\x0c\x12\r\n\x05\x64\x65pth\x18\x03 \x01(\x0c\x1a\x1c\n\x06\x44\x61mage\x12\x12\n\nis_damaged\x18\x01 \x01(\x08\x1a\x37\n\x12LaneCenterDistance\x12\x0f\n\x07lane_id\x18\x01 \x01(\t\x12\x10\n\x08\x64istance\x18\x02 \x01(\x02\x1a\x30\n\x0e\x43\x61rToLaneAngle\x12\x0f\n\x07lane_id\x18\x01 \x01(\t\x12\r\n\x05\x61ngle\x18\x02 \x01(\x02\x1a\x1d\n\x0b\x42oundingBox\x12\x0e\n\x06points\x18\x01 \x03(\x02\x1a\x18\n\x05\x45rror\x12\x0f\n\x07message\x18\x01 \x01(\tB\x06\n\x04\x64\x61ta\x1a?\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.DataResponse.Data:\x02\x38\x01\"\x91\x02\n\x07\x43ontrol\x12\'\n\tavCommand\x18\x01 \x01(\x0b\x32\x12.Control.AvCommandH\x00\x12)\n\nsimCommand\x18\x02 \x01(\x0b\x32\x13.Control.SimCommandH\x00\x1a=\n\tAvCommand\x12\x12\n\naccelerate\x18\x01 \x01(\x01\x12\r\n\x05steer\x18\x02 \x01(\x01\x12\r\n\x05\x62rake\x18\x03 \x01(\x01\x1ah\n\nSimCommand\x12,\n\x07\x63ommand\x18\x01 \x01(\x0e\x32\x1b.Control.SimCommand.Command\",\n\x07\x43ommand\x12\x0b\n\x07SUCCEED\x10\x00\x12\x08\n\x04\x46\x41IL\x10\x01\x12\n\n\x06\x43\x41NCEL\x10\x02\x42\t\n\x07\x63ommand\"L\n\x12VerificationResult\x12\x14\n\x0cprecondition\x18\x01 \x01(\t\x12\x0f\n\x07\x66\x61ilure\x18\x02 \x01(\t\x12\x0f\n\x07success\x18\x03 \x01(\t\"\x18\n\tVehicleID\x12\x0b\n\x03vid\x18\x01 \x01(\t\"\x1a\n\nVehicleIDs\x12\x0c\n\x04vids\x18\x01 \x03(\t\"\x1b\n\x0cSimulationID\x12\x0b\n\x03sid\x18\x01 \x01(\t\"\x1d\n\rSimulationIDs\x12\x0c\n\x04sids\x18\x01 \x03(\t\"\x88\x02\n\x10SubmissionResult\x12/\n\x06result\x18\x01 \x01(\x0b\x32\x1d.SubmissionResult.SubmissionsH\x00\x12\x18\n\x07message\x18\x02 \x01(\x0b\x32\x05.VoidH\x00\x1a\x95\x01\n\x0bSubmissions\x12\x43\n\x0bsubmissions\x18\x01 \x03(\x0b\x32..SubmissionResult.Submissions.SubmissionsEntry\x1a\x41\n\x10SubmissionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1c\n\x05value\x18\x02 \x01(\x0b\x32\r.SimulationID:\x02\x38\x01\x42\x11\n\x0fmay_submissions\" \n\x10SimulationNodeID\x12\x0c\n\x04snid\x18\x01 \x01(\t\"\x12\n\x03Num\x12\x0b\n\x03num\x18\x01 \x01(\x05\"\x15\n\x04\x42ool\x12\r\n\x05value\x18\x01 \x01(\x08\"\x99\x01\n\x10SimStateResponse\x12)\n\x05state\x18\x01 \x01(\x0e\x32\x1a.SimStateResponse.SimState\"Z\n\x08SimState\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x0b\n\x07RUNNING\x10\x01\x12\x0c\n\x08\x46INISHED\x10\x02\x12\x0c\n\x08\x43\x41NCELED\x10\x03\x12\x0b\n\x07TIMEOUT\x10\x04\x12\x0b\n\x07UNKNOWN\x10\x05\"|\n\nTestResult\x12\"\n\x06result\x18\x01 \x01(\x0e\x32\x12.TestResult.Result\"J\n\x06Result\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\r\n\tSUCCEEDED\x10\x01\x12\n\n\x06\x46\x41ILED\x10\x02\x12\x0b\n\x07SKIPPED\x10\x03\x12\x0b\n\x07UNKNOWN\x10\x04\"\x17\n\x04Void\x12\x0f\n\x07message\x18\x01 \x01(\t\"*\n\x04User\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\tB\x03\x90\x01\x00\x62\x06proto3')
)



_CONTROL_SIMCOMMAND_COMMAND = _descriptor.EnumDescriptor(
  name='Command',
  full_name='Control.SimCommand.Command',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCEED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAIL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CANCEL', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1299,
  serialized_end=1343,
)
_sym_db.RegisterEnumDescriptor(_CONTROL_SIMCOMMAND_COMMAND)

_SIMSTATERESPONSE_SIMSTATE = _descriptor.EnumDescriptor(
  name='SimState',
  full_name='SimStateResponse.SimState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DEFAULT', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RUNNING', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FINISHED', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CANCELED', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TIMEOUT', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1956,
  serialized_end=2046,
)
_sym_db.RegisterEnumDescriptor(_SIMSTATERESPONSE_SIMSTATE)

_TESTRESULT_RESULT = _descriptor.EnumDescriptor(
  name='Result',
  full_name='TestResult.Result',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DEFAULT', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUCCEEDED', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SKIPPED', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2098,
  serialized_end=2172,
)
_sym_db.RegisterEnumDescriptor(_TESTRESULT_RESULT)


_DATAREQUEST = _descriptor.Descriptor(
  name='DataRequest',
  full_name='DataRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='request_ids', full_name='DataRequest.request_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=62,
)


_DATARESPONSE_DATA_POSITION = _descriptor.Descriptor(
  name='Position',
  full_name='DataResponse.Data.Position',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='DataResponse.Data.Position.x', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='DataResponse.Data.Position.y', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=639,
  serialized_end=671,
)

_DATARESPONSE_DATA_SPEED = _descriptor.Descriptor(
  name='Speed',
  full_name='DataResponse.Data.Speed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='speed', full_name='DataResponse.Data.Speed.speed', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=673,
  serialized_end=695,
)

_DATARESPONSE_DATA_STEERINGANGLE = _descriptor.Descriptor(
  name='SteeringAngle',
  full_name='DataResponse.Data.SteeringAngle',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='angle', full_name='DataResponse.Data.SteeringAngle.angle', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=697,
  serialized_end=727,
)

_DATARESPONSE_DATA_LIDAR = _descriptor.Descriptor(
  name='Lidar',
  full_name='DataResponse.Data.Lidar',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='DataResponse.Data.Lidar.points', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=729,
  serialized_end=752,
)

_DATARESPONSE_DATA_CAMERA = _descriptor.Descriptor(
  name='Camera',
  full_name='DataResponse.Data.Camera',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='color', full_name='DataResponse.Data.Camera.color', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotated', full_name='DataResponse.Data.Camera.annotated', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='depth', full_name='DataResponse.Data.Camera.depth', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=754,
  serialized_end=811,
)

_DATARESPONSE_DATA_DAMAGE = _descriptor.Descriptor(
  name='Damage',
  full_name='DataResponse.Data.Damage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='is_damaged', full_name='DataResponse.Data.Damage.is_damaged', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=813,
  serialized_end=841,
)

_DATARESPONSE_DATA_LANECENTERDISTANCE = _descriptor.Descriptor(
  name='LaneCenterDistance',
  full_name='DataResponse.Data.LaneCenterDistance',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lane_id', full_name='DataResponse.Data.LaneCenterDistance.lane_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='distance', full_name='DataResponse.Data.LaneCenterDistance.distance', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=843,
  serialized_end=898,
)

_DATARESPONSE_DATA_CARTOLANEANGLE = _descriptor.Descriptor(
  name='CarToLaneAngle',
  full_name='DataResponse.Data.CarToLaneAngle',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lane_id', full_name='DataResponse.Data.CarToLaneAngle.lane_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='angle', full_name='DataResponse.Data.CarToLaneAngle.angle', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=900,
  serialized_end=948,
)

_DATARESPONSE_DATA_BOUNDINGBOX = _descriptor.Descriptor(
  name='BoundingBox',
  full_name='DataResponse.Data.BoundingBox',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='DataResponse.Data.BoundingBox.points', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=950,
  serialized_end=979,
)

_DATARESPONSE_DATA_ERROR = _descriptor.Descriptor(
  name='Error',
  full_name='DataResponse.Data.Error',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='DataResponse.Data.Error.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=981,
  serialized_end=1005,
)

_DATARESPONSE_DATA = _descriptor.Descriptor(
  name='Data',
  full_name='DataResponse.Data',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='position', full_name='DataResponse.Data.position', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='speed', full_name='DataResponse.Data.speed', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='angle', full_name='DataResponse.Data.angle', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lidar', full_name='DataResponse.Data.lidar', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='camera', full_name='DataResponse.Data.camera', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='damage', full_name='DataResponse.Data.damage', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lane_center_distance', full_name='DataResponse.Data.lane_center_distance', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='car_to_lane_angle', full_name='DataResponse.Data.car_to_lane_angle', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bounding_box', full_name='DataResponse.Data.bounding_box', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='DataResponse.Data.error', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DATARESPONSE_DATA_POSITION, _DATARESPONSE_DATA_SPEED, _DATARESPONSE_DATA_STEERINGANGLE, _DATARESPONSE_DATA_LIDAR, _DATARESPONSE_DATA_CAMERA, _DATARESPONSE_DATA_DAMAGE, _DATARESPONSE_DATA_LANECENTERDISTANCE, _DATARESPONSE_DATA_CARTOLANEANGLE, _DATARESPONSE_DATA_BOUNDINGBOX, _DATARESPONSE_DATA_ERROR, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='DataResponse.Data.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=121,
  serialized_end=1013,
)

_DATARESPONSE_DATAENTRY = _descriptor.Descriptor(
  name='DataEntry',
  full_name='DataResponse.DataEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='DataResponse.DataEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='DataResponse.DataEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1015,
  serialized_end=1078,
)

_DATARESPONSE = _descriptor.Descriptor(
  name='DataResponse',
  full_name='DataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='DataResponse.data', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DATARESPONSE_DATA, _DATARESPONSE_DATAENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=1078,
)


_CONTROL_AVCOMMAND = _descriptor.Descriptor(
  name='AvCommand',
  full_name='Control.AvCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='accelerate', full_name='Control.AvCommand.accelerate', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='steer', full_name='Control.AvCommand.steer', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='brake', full_name='Control.AvCommand.brake', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1176,
  serialized_end=1237,
)

_CONTROL_SIMCOMMAND = _descriptor.Descriptor(
  name='SimCommand',
  full_name='Control.SimCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='command', full_name='Control.SimCommand.command', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _CONTROL_SIMCOMMAND_COMMAND,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1239,
  serialized_end=1343,
)

_CONTROL = _descriptor.Descriptor(
  name='Control',
  full_name='Control',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='avCommand', full_name='Control.avCommand', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='simCommand', full_name='Control.simCommand', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONTROL_AVCOMMAND, _CONTROL_SIMCOMMAND, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='command', full_name='Control.command',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1081,
  serialized_end=1354,
)


_VERIFICATIONRESULT = _descriptor.Descriptor(
  name='VerificationResult',
  full_name='VerificationResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='precondition', full_name='VerificationResult.precondition', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='failure', full_name='VerificationResult.failure', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='success', full_name='VerificationResult.success', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1356,
  serialized_end=1432,
)


_VEHICLEID = _descriptor.Descriptor(
  name='VehicleID',
  full_name='VehicleID',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vid', full_name='VehicleID.vid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1434,
  serialized_end=1458,
)


_VEHICLEIDS = _descriptor.Descriptor(
  name='VehicleIDs',
  full_name='VehicleIDs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vids', full_name='VehicleIDs.vids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1460,
  serialized_end=1486,
)


_SIMULATIONID = _descriptor.Descriptor(
  name='SimulationID',
  full_name='SimulationID',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sid', full_name='SimulationID.sid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1488,
  serialized_end=1515,
)


_SIMULATIONIDS = _descriptor.Descriptor(
  name='SimulationIDs',
  full_name='SimulationIDs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sids', full_name='SimulationIDs.sids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1517,
  serialized_end=1546,
)


_SUBMISSIONRESULT_SUBMISSIONS_SUBMISSIONSENTRY = _descriptor.Descriptor(
  name='SubmissionsEntry',
  full_name='SubmissionResult.Submissions.SubmissionsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='SubmissionResult.Submissions.SubmissionsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='SubmissionResult.Submissions.SubmissionsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1729,
  serialized_end=1794,
)

_SUBMISSIONRESULT_SUBMISSIONS = _descriptor.Descriptor(
  name='Submissions',
  full_name='SubmissionResult.Submissions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='submissions', full_name='SubmissionResult.Submissions.submissions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SUBMISSIONRESULT_SUBMISSIONS_SUBMISSIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1645,
  serialized_end=1794,
)

_SUBMISSIONRESULT = _descriptor.Descriptor(
  name='SubmissionResult',
  full_name='SubmissionResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='SubmissionResult.result', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='SubmissionResult.message', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SUBMISSIONRESULT_SUBMISSIONS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='may_submissions', full_name='SubmissionResult.may_submissions',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1549,
  serialized_end=1813,
)


_SIMULATIONNODEID = _descriptor.Descriptor(
  name='SimulationNodeID',
  full_name='SimulationNodeID',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='snid', full_name='SimulationNodeID.snid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1815,
  serialized_end=1847,
)


_NUM = _descriptor.Descriptor(
  name='Num',
  full_name='Num',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='num', full_name='Num.num', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1849,
  serialized_end=1867,
)


_BOOL = _descriptor.Descriptor(
  name='Bool',
  full_name='Bool',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='Bool.value', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1869,
  serialized_end=1890,
)


_SIMSTATERESPONSE = _descriptor.Descriptor(
  name='SimStateResponse',
  full_name='SimStateResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='SimStateResponse.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SIMSTATERESPONSE_SIMSTATE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1893,
  serialized_end=2046,
)


_TESTRESULT = _descriptor.Descriptor(
  name='TestResult',
  full_name='TestResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='TestResult.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TESTRESULT_RESULT,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2048,
  serialized_end=2172,
)


_VOID = _descriptor.Descriptor(
  name='Void',
  full_name='Void',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='Void.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2174,
  serialized_end=2197,
)


_USER = _descriptor.Descriptor(
  name='User',
  full_name='User',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='username', full_name='User.username', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='User.password', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2199,
  serialized_end=2241,
)

_DATARESPONSE_DATA_POSITION.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA_SPEED.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA_STEERINGANGLE.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA_LIDAR.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA_CAMERA.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA_DAMAGE.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA_LANECENTERDISTANCE.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA_CARTOLANEANGLE.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA_BOUNDINGBOX.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA_ERROR.containing_type = _DATARESPONSE_DATA
_DATARESPONSE_DATA.fields_by_name['position'].message_type = _DATARESPONSE_DATA_POSITION
_DATARESPONSE_DATA.fields_by_name['speed'].message_type = _DATARESPONSE_DATA_SPEED
_DATARESPONSE_DATA.fields_by_name['angle'].message_type = _DATARESPONSE_DATA_STEERINGANGLE
_DATARESPONSE_DATA.fields_by_name['lidar'].message_type = _DATARESPONSE_DATA_LIDAR
_DATARESPONSE_DATA.fields_by_name['camera'].message_type = _DATARESPONSE_DATA_CAMERA
_DATARESPONSE_DATA.fields_by_name['damage'].message_type = _DATARESPONSE_DATA_DAMAGE
_DATARESPONSE_DATA.fields_by_name['lane_center_distance'].message_type = _DATARESPONSE_DATA_LANECENTERDISTANCE
_DATARESPONSE_DATA.fields_by_name['car_to_lane_angle'].message_type = _DATARESPONSE_DATA_CARTOLANEANGLE
_DATARESPONSE_DATA.fields_by_name['bounding_box'].message_type = _DATARESPONSE_DATA_BOUNDINGBOX
_DATARESPONSE_DATA.fields_by_name['error'].message_type = _DATARESPONSE_DATA_ERROR
_DATARESPONSE_DATA.containing_type = _DATARESPONSE
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['position'])
_DATARESPONSE_DATA.fields_by_name['position'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['speed'])
_DATARESPONSE_DATA.fields_by_name['speed'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['angle'])
_DATARESPONSE_DATA.fields_by_name['angle'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['lidar'])
_DATARESPONSE_DATA.fields_by_name['lidar'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['camera'])
_DATARESPONSE_DATA.fields_by_name['camera'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['damage'])
_DATARESPONSE_DATA.fields_by_name['damage'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['lane_center_distance'])
_DATARESPONSE_DATA.fields_by_name['lane_center_distance'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['car_to_lane_angle'])
_DATARESPONSE_DATA.fields_by_name['car_to_lane_angle'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['bounding_box'])
_DATARESPONSE_DATA.fields_by_name['bounding_box'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATA.oneofs_by_name['data'].fields.append(
  _DATARESPONSE_DATA.fields_by_name['error'])
_DATARESPONSE_DATA.fields_by_name['error'].containing_oneof = _DATARESPONSE_DATA.oneofs_by_name['data']
_DATARESPONSE_DATAENTRY.fields_by_name['value'].message_type = _DATARESPONSE_DATA
_DATARESPONSE_DATAENTRY.containing_type = _DATARESPONSE
_DATARESPONSE.fields_by_name['data'].message_type = _DATARESPONSE_DATAENTRY
_CONTROL_AVCOMMAND.containing_type = _CONTROL
_CONTROL_SIMCOMMAND.fields_by_name['command'].enum_type = _CONTROL_SIMCOMMAND_COMMAND
_CONTROL_SIMCOMMAND.containing_type = _CONTROL
_CONTROL_SIMCOMMAND_COMMAND.containing_type = _CONTROL_SIMCOMMAND
_CONTROL.fields_by_name['avCommand'].message_type = _CONTROL_AVCOMMAND
_CONTROL.fields_by_name['simCommand'].message_type = _CONTROL_SIMCOMMAND
_CONTROL.oneofs_by_name['command'].fields.append(
  _CONTROL.fields_by_name['avCommand'])
_CONTROL.fields_by_name['avCommand'].containing_oneof = _CONTROL.oneofs_by_name['command']
_CONTROL.oneofs_by_name['command'].fields.append(
  _CONTROL.fields_by_name['simCommand'])
_CONTROL.fields_by_name['simCommand'].containing_oneof = _CONTROL.oneofs_by_name['command']
_SUBMISSIONRESULT_SUBMISSIONS_SUBMISSIONSENTRY.fields_by_name['value'].message_type = _SIMULATIONID
_SUBMISSIONRESULT_SUBMISSIONS_SUBMISSIONSENTRY.containing_type = _SUBMISSIONRESULT_SUBMISSIONS
_SUBMISSIONRESULT_SUBMISSIONS.fields_by_name['submissions'].message_type = _SUBMISSIONRESULT_SUBMISSIONS_SUBMISSIONSENTRY
_SUBMISSIONRESULT_SUBMISSIONS.containing_type = _SUBMISSIONRESULT
_SUBMISSIONRESULT.fields_by_name['result'].message_type = _SUBMISSIONRESULT_SUBMISSIONS
_SUBMISSIONRESULT.fields_by_name['message'].message_type = _VOID
_SUBMISSIONRESULT.oneofs_by_name['may_submissions'].fields.append(
  _SUBMISSIONRESULT.fields_by_name['result'])
_SUBMISSIONRESULT.fields_by_name['result'].containing_oneof = _SUBMISSIONRESULT.oneofs_by_name['may_submissions']
_SUBMISSIONRESULT.oneofs_by_name['may_submissions'].fields.append(
  _SUBMISSIONRESULT.fields_by_name['message'])
_SUBMISSIONRESULT.fields_by_name['message'].containing_oneof = _SUBMISSIONRESULT.oneofs_by_name['may_submissions']
_SIMSTATERESPONSE.fields_by_name['state'].enum_type = _SIMSTATERESPONSE_SIMSTATE
_SIMSTATERESPONSE_SIMSTATE.containing_type = _SIMSTATERESPONSE
_TESTRESULT.fields_by_name['result'].enum_type = _TESTRESULT_RESULT
_TESTRESULT_RESULT.containing_type = _TESTRESULT
DESCRIPTOR.message_types_by_name['DataRequest'] = _DATAREQUEST
DESCRIPTOR.message_types_by_name['DataResponse'] = _DATARESPONSE
DESCRIPTOR.message_types_by_name['Control'] = _CONTROL
DESCRIPTOR.message_types_by_name['VerificationResult'] = _VERIFICATIONRESULT
DESCRIPTOR.message_types_by_name['VehicleID'] = _VEHICLEID
DESCRIPTOR.message_types_by_name['VehicleIDs'] = _VEHICLEIDS
DESCRIPTOR.message_types_by_name['SimulationID'] = _SIMULATIONID
DESCRIPTOR.message_types_by_name['SimulationIDs'] = _SIMULATIONIDS
DESCRIPTOR.message_types_by_name['SubmissionResult'] = _SUBMISSIONRESULT
DESCRIPTOR.message_types_by_name['SimulationNodeID'] = _SIMULATIONNODEID
DESCRIPTOR.message_types_by_name['Num'] = _NUM
DESCRIPTOR.message_types_by_name['Bool'] = _BOOL
DESCRIPTOR.message_types_by_name['SimStateResponse'] = _SIMSTATERESPONSE
DESCRIPTOR.message_types_by_name['TestResult'] = _TESTRESULT
DESCRIPTOR.message_types_by_name['Void'] = _VOID
DESCRIPTOR.message_types_by_name['User'] = _USER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DataRequest = _reflection.GeneratedProtocolMessageType('DataRequest', (_message.Message,), dict(
  DESCRIPTOR = _DATAREQUEST,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:DataRequest)
  ))
_sym_db.RegisterMessage(DataRequest)

DataResponse = _reflection.GeneratedProtocolMessageType('DataResponse', (_message.Message,), dict(

  Data = _reflection.GeneratedProtocolMessageType('Data', (_message.Message,), dict(

    Position = _reflection.GeneratedProtocolMessageType('Position', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_POSITION,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.Position)
      ))
    ,

    Speed = _reflection.GeneratedProtocolMessageType('Speed', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_SPEED,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.Speed)
      ))
    ,

    SteeringAngle = _reflection.GeneratedProtocolMessageType('SteeringAngle', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_STEERINGANGLE,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.SteeringAngle)
      ))
    ,

    Lidar = _reflection.GeneratedProtocolMessageType('Lidar', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_LIDAR,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.Lidar)
      ))
    ,

    Camera = _reflection.GeneratedProtocolMessageType('Camera', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_CAMERA,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.Camera)
      ))
    ,

    Damage = _reflection.GeneratedProtocolMessageType('Damage', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_DAMAGE,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.Damage)
      ))
    ,

    LaneCenterDistance = _reflection.GeneratedProtocolMessageType('LaneCenterDistance', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_LANECENTERDISTANCE,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.LaneCenterDistance)
      ))
    ,

    CarToLaneAngle = _reflection.GeneratedProtocolMessageType('CarToLaneAngle', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_CARTOLANEANGLE,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.CarToLaneAngle)
      ))
    ,

    BoundingBox = _reflection.GeneratedProtocolMessageType('BoundingBox', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_BOUNDINGBOX,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.BoundingBox)
      ))
    ,

    Error = _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), dict(
      DESCRIPTOR = _DATARESPONSE_DATA_ERROR,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:DataResponse.Data.Error)
      ))
    ,
    DESCRIPTOR = _DATARESPONSE_DATA,
    __module__ = 'aiExchangeMessages_pb2'
    # @@protoc_insertion_point(class_scope:DataResponse.Data)
    ))
  ,

  DataEntry = _reflection.GeneratedProtocolMessageType('DataEntry', (_message.Message,), dict(
    DESCRIPTOR = _DATARESPONSE_DATAENTRY,
    __module__ = 'aiExchangeMessages_pb2'
    # @@protoc_insertion_point(class_scope:DataResponse.DataEntry)
    ))
  ,
  DESCRIPTOR = _DATARESPONSE,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:DataResponse)
  ))
_sym_db.RegisterMessage(DataResponse)
_sym_db.RegisterMessage(DataResponse.Data)
_sym_db.RegisterMessage(DataResponse.Data.Position)
_sym_db.RegisterMessage(DataResponse.Data.Speed)
_sym_db.RegisterMessage(DataResponse.Data.SteeringAngle)
_sym_db.RegisterMessage(DataResponse.Data.Lidar)
_sym_db.RegisterMessage(DataResponse.Data.Camera)
_sym_db.RegisterMessage(DataResponse.Data.Damage)
_sym_db.RegisterMessage(DataResponse.Data.LaneCenterDistance)
_sym_db.RegisterMessage(DataResponse.Data.CarToLaneAngle)
_sym_db.RegisterMessage(DataResponse.Data.BoundingBox)
_sym_db.RegisterMessage(DataResponse.Data.Error)
_sym_db.RegisterMessage(DataResponse.DataEntry)

Control = _reflection.GeneratedProtocolMessageType('Control', (_message.Message,), dict(

  AvCommand = _reflection.GeneratedProtocolMessageType('AvCommand', (_message.Message,), dict(
    DESCRIPTOR = _CONTROL_AVCOMMAND,
    __module__ = 'aiExchangeMessages_pb2'
    # @@protoc_insertion_point(class_scope:Control.AvCommand)
    ))
  ,

  SimCommand = _reflection.GeneratedProtocolMessageType('SimCommand', (_message.Message,), dict(
    DESCRIPTOR = _CONTROL_SIMCOMMAND,
    __module__ = 'aiExchangeMessages_pb2'
    # @@protoc_insertion_point(class_scope:Control.SimCommand)
    ))
  ,
  DESCRIPTOR = _CONTROL,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:Control)
  ))
_sym_db.RegisterMessage(Control)
_sym_db.RegisterMessage(Control.AvCommand)
_sym_db.RegisterMessage(Control.SimCommand)

VerificationResult = _reflection.GeneratedProtocolMessageType('VerificationResult', (_message.Message,), dict(
  DESCRIPTOR = _VERIFICATIONRESULT,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:VerificationResult)
  ))
_sym_db.RegisterMessage(VerificationResult)

VehicleID = _reflection.GeneratedProtocolMessageType('VehicleID', (_message.Message,), dict(
  DESCRIPTOR = _VEHICLEID,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:VehicleID)
  ))
_sym_db.RegisterMessage(VehicleID)

VehicleIDs = _reflection.GeneratedProtocolMessageType('VehicleIDs', (_message.Message,), dict(
  DESCRIPTOR = _VEHICLEIDS,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:VehicleIDs)
  ))
_sym_db.RegisterMessage(VehicleIDs)

SimulationID = _reflection.GeneratedProtocolMessageType('SimulationID', (_message.Message,), dict(
  DESCRIPTOR = _SIMULATIONID,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:SimulationID)
  ))
_sym_db.RegisterMessage(SimulationID)

SimulationIDs = _reflection.GeneratedProtocolMessageType('SimulationIDs', (_message.Message,), dict(
  DESCRIPTOR = _SIMULATIONIDS,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:SimulationIDs)
  ))
_sym_db.RegisterMessage(SimulationIDs)

SubmissionResult = _reflection.GeneratedProtocolMessageType('SubmissionResult', (_message.Message,), dict(

  Submissions = _reflection.GeneratedProtocolMessageType('Submissions', (_message.Message,), dict(

    SubmissionsEntry = _reflection.GeneratedProtocolMessageType('SubmissionsEntry', (_message.Message,), dict(
      DESCRIPTOR = _SUBMISSIONRESULT_SUBMISSIONS_SUBMISSIONSENTRY,
      __module__ = 'aiExchangeMessages_pb2'
      # @@protoc_insertion_point(class_scope:SubmissionResult.Submissions.SubmissionsEntry)
      ))
    ,
    DESCRIPTOR = _SUBMISSIONRESULT_SUBMISSIONS,
    __module__ = 'aiExchangeMessages_pb2'
    # @@protoc_insertion_point(class_scope:SubmissionResult.Submissions)
    ))
  ,
  DESCRIPTOR = _SUBMISSIONRESULT,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:SubmissionResult)
  ))
_sym_db.RegisterMessage(SubmissionResult)
_sym_db.RegisterMessage(SubmissionResult.Submissions)
_sym_db.RegisterMessage(SubmissionResult.Submissions.SubmissionsEntry)

SimulationNodeID = _reflection.GeneratedProtocolMessageType('SimulationNodeID', (_message.Message,), dict(
  DESCRIPTOR = _SIMULATIONNODEID,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:SimulationNodeID)
  ))
_sym_db.RegisterMessage(SimulationNodeID)

Num = _reflection.GeneratedProtocolMessageType('Num', (_message.Message,), dict(
  DESCRIPTOR = _NUM,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:Num)
  ))
_sym_db.RegisterMessage(Num)

Bool = _reflection.GeneratedProtocolMessageType('Bool', (_message.Message,), dict(
  DESCRIPTOR = _BOOL,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:Bool)
  ))
_sym_db.RegisterMessage(Bool)

SimStateResponse = _reflection.GeneratedProtocolMessageType('SimStateResponse', (_message.Message,), dict(
  DESCRIPTOR = _SIMSTATERESPONSE,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:SimStateResponse)
  ))
_sym_db.RegisterMessage(SimStateResponse)

TestResult = _reflection.GeneratedProtocolMessageType('TestResult', (_message.Message,), dict(
  DESCRIPTOR = _TESTRESULT,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:TestResult)
  ))
_sym_db.RegisterMessage(TestResult)

Void = _reflection.GeneratedProtocolMessageType('Void', (_message.Message,), dict(
  DESCRIPTOR = _VOID,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:Void)
  ))
_sym_db.RegisterMessage(Void)

User = _reflection.GeneratedProtocolMessageType('User', (_message.Message,), dict(
  DESCRIPTOR = _USER,
  __module__ = 'aiExchangeMessages_pb2'
  # @@protoc_insertion_point(class_scope:User)
  ))
_sym_db.RegisterMessage(User)


DESCRIPTOR._options = None
_DATARESPONSE_DATAENTRY._options = None
_SUBMISSIONRESULT_SUBMISSIONS_SUBMISSIONSENTRY._options = None
# @@protoc_insertion_point(module_scope)



from http.client import HTTPResponse
from typing import List, Callable, Union, Tuple, Any, Dict, AnyStr, Optional

from flask import Response

from drivebuildclient.aiExchangeMessages_pb2 import SimulationID, VehicleID, SimulationNodeID

AnyResponse = Union[Response, Tuple[Any, int]]


# FIXME Merge methods into only two methods?


def do_post_request(host: str, port: int, address: str, content: bytes) -> HTTPResponse:
    from http.client import HTTPConnection
    connection = HTTPConnection(host=host, port=port)
    connection.request("POST", address, body=content, headers={"content-type": "application/x-protobuf"})
    return connection.getresponse()


def do_get_request(host: str, port: int, address: str, params: Dict[str, AnyStr]) -> HTTPResponse:
    """
    :return: The response object of the request
    """
    from urllib.parse import urlencode
    from http.client import HTTPConnection
    connection = HTTPConnection(host=host, port=port)
    connection.request("GET", address + "?" + urlencode(params), headers={"content-type": "application/x-protobuf"})
    return connection.getresponse()


def process_get_request(min_params: List[str], on_parameter_available: Callable[[], AnyResponse]) -> AnyResponse:
    """
    This stub is designed for GET requests.
    """
    from flask import request
    missing_params = list(filter(lambda p: p not in request.args, min_params))
    if missing_params:
        return Response(response="The request misses one of the parameters [\"" + "\", ".join(missing_params) + "\"]",
                        status=400, mimetype="text/plain")
    else:
        return on_parameter_available()


def do_mixed_request(host: str, port: int, address: str, params: Dict[str, AnyStr], content: bytes) -> HTTPResponse:
    from urllib.parse import urlencode
    from http.client import HTTPConnection
    connection = HTTPConnection(host=host, port=port)
    connection.request("POST", address + "?" + urlencode(params), body=content,
                       headers={"content-type": "application/x-protobuf"})
    return connection.getresponse()


def process_mixed_request(min_params: List[str], on_parameter_available: Callable[[], AnyResponse]) -> AnyResponse:
    from flask import request
    # FIXME Any difference to processing GET requests?
    missing_params = list(filter(lambda p: p not in request.args, min_params))
    if missing_params:
        return Response(response="The request misses one of the parameters [\"" + "\", ".join(missing_params) + "\"]",
                        status=400, mimetype="text/plain")
    else:
        return on_parameter_available()


def extract_sid() -> Tuple[Optional[bytes], Optional[SimulationID]]:
    from flask import request
    serialized_sid_b = request.args.get("sid", default=None)
    if serialized_sid_b:
        serialized_sid = serialized_sid_b.encode()
        sid = SimulationID()
        sid.ParseFromString(serialized_sid)
        return serialized_sid, sid
    else:
        return None, None


def extract_vid() -> Tuple[Optional[bytes], Optional[VehicleID]]:
    from flask import request
    serialized_vid_b = request.args.get("vid", default=None)
    if serialized_vid_b:
        serialized_vid = serialized_vid_b.encode()
        vid = VehicleID()
        vid.ParseFromString(serialized_vid)
        return serialized_vid, vid
    else:
        return None, None


def extract_snid() -> Tuple[Optional[bytes], Optional[SimulationNodeID]]:
    from flask import request
    serialized_snid_b = request.args.get("snid", default=None)
    if serialized_snid_b:
        serialized_snid = serialized_snid_b.encode()
        snid = SimulationNodeID()
        snid.ParseFromString(serialized_snid)
        return serialized_snid, snid
    else:
        return None, None



from http.client import HTTPResponse
from pathlib import Path
from typing import Optional, List, Tuple

from drivebuildclient.aiExchangeMessages_pb2 import VehicleID, SimulationID, SimulationIDs, TestResult, \
    SubmissionResult, User, DataResponse


class AIExchangeService:
    from drivebuildclient.aiExchangeMessages_pb2 import SimStateResponse, DataRequest, Control, Void

    def __init__(self, host: str, port: int):
        self.host = host
        self.port = port

    @staticmethod
    def _print_error(response: HTTPResponse) -> None:
        from drivebuildclient.common import eprint
        eprint("Response status: " + str(response.status))
        eprint("Reason: " + response.reason)
        eprint("Messsage:")
        eprint(b"\n".join(response.readlines()))

    def wait_for_simulator_request(self, sid: SimulationID, vid: VehicleID) -> SimStateResponse.SimState:
        """
        Waits for the simulation with ID sid to request the car with ID vid. This call blocks until the simulation
        requests the appropriate car in the given simulation.
        :param sid: The ID of the simulation the vehicle is included in.
        :param vid: The ID of the vehicle in the simulation to wait for.
        :return: The current state of the simulation at the point when the call to this function returns. The return
        value should be used to check whether the simulation is still running. Another vehicle or the even user may have
        stopped the simulation.
        """
        from drivebuildclient.aiExchangeMessages_pb2 import SimStateResponse
        from drivebuildclient.httpUtil import do_get_request
        response = do_get_request(self.host, self.port, "/ai/waitForSimulatorRequest", {
            "sid": sid.SerializeToString(),
            "vid": vid.SerializeToString()
        })
        if response.status == 200:
            result = b"".join(response.readlines())
            sim_state = SimStateResponse()
            sim_state.ParseFromString(result)
            return sim_state.state
        else:
            AIExchangeService._print_error(response)

    def request_data(self, sid: SimulationID, vid: VehicleID, request: DataRequest) -> DataResponse:
        """
        Request data of a certain vehicle contained by a certain simulation.
        :param sid: The ID of the simulation the vehicle to request data about is part of.
        :param vid: The ID of the vehicle to get collected data from.
        :param request: The types of data to be requested about the given vehicle. A DataRequest object is build like
        the following:
        request = DataRequest()
        request.request_ids.extend(["id_1", "id_2",..., "id_n"])
        NOTE: You have to use extend(...)! An assignment like request.request_ids = [...] will not work due to the
        implementation of Googles protobuffer.
        :return: The data the simulation collected about the given vehicle. The way of accessing the data is dependant
        on the type of data you requested. To find out how to access the data properly you should set a break point and
        checkout the content of the returned value using a debugger.
        """
        from drivebuildclient.aiExchangeMessages_pb2 import DataResponse
        from drivebuildclient.httpUtil import do_get_request
        response = do_get_request(self.host, self.port, "/ai/requestData", {
            "request": request.SerializeToString(),
            "sid": sid.SerializeToString(),
            "vid": vid.SerializeToString()
        })
        if response.status == 200:
            result = b"".join(response.readlines())
            data_response = DataResponse()
            data_response.ParseFromString(result)
            return data_response
        else:
            AIExchangeService._print_error(response)

    def control(self, sid: SimulationID, vid: VehicleID, commands: Control) -> Optional[Void]:
        """
        Control the simulation or a certain vehicle in the simulation.
        :param sid: The ID the simulation either to control or containing th vehicle to control.
        :param vid: The ID of the vehicle to possibly control.
        :param commands: The command either controlling a simulation or a vehicle in a simualtion. To define a command
        controlling a simulation you can use commands like:
        control = Control()
        control.simCommand.command = Control.SimCommand.Command.SUCCEED  # Force simulation to succeed
        control.simCommand.command = Control.SimCommand.Command.FAIL  # Force simulation to fail
        control.simCommand.command = Control.SimCommand.Command.CANCEL  # Force simulation to be cancelled/skipped
        For controlling a vehicle you have to define steering, acceleration and brake values:
        control = Control()
        control.avCommand.accelerate = <Acceleration intensity having a value between 0.0 and 1.0>
        control.avCommand.steer = <A steering value between -1.0 and 1.0 (Negative value steers left; a positive one
        steers right)>
        control.avCommand.brake = <Brake intensity having a value between 0.0 and 1.0>
        :return: A Void object possibly containing a info message.
        """
        from drivebuildclient.httpUtil import do_mixed_request
        response = do_mixed_request(self.host, self.port, "/ai/control", {
            "sid": sid.SerializeToString(),
            "vid": vid.SerializeToString()
        }, commands.SerializeToString())
        if response.status == 200:
            void = Void()
            void.ParseFromString(b"".join(response.readlines()))
            return void
        else:
            AIExchangeService._print_error(response)

    def control_sim(self, sid: SimulationID, result: TestResult) -> Optional[Void]:
        """
        Force a simulation to end having the given result.
        :param sid: The simulation to control.
        :param result: The test result to be set to the simulation.
        :return: A Void object possibly containing a info message.
        """
        from drivebuildclient.httpUtil import do_get_request
        response = do_get_request(self.host, self.port, "/sim/stop", {
            "sid": sid.SerializeToString(),
            "result": result.SerializeToString()
        })
        if response.status == 200:
            void = Void()
            void.ParseFromString(b"".join(response.readlines()))
            return void
        else:
            AIExchangeService._print_error(response)

    def get_status(self, sid: SimulationID) -> str:
        """
        Check the status of the given simulation.
        :param sid: The simulation to get the status of.
        :return: A string representing the status of the simulation like RUNNING, FINISHED or ERRORED.
        """
        from drivebuildclient.httpUtil import do_get_request
        import dill as pickle
        response = do_get_request(self.host, self.port, "/stats/status", {
            "sid": sid.SerializeToString()
        })
        if response.status == 200:
            return pickle.loads(b"".join(response.readlines()))
        else:
            AIExchangeService._print_error(response)
            return "Status could not be determined."

    def get_result(self, sid: SimulationID) -> str:
        """
        Get the test result of the given simulation. This call blocks until the test result of the simulation is known.
        :param sid: The simulation to get the test result of.
        :return: The current test result of the given simulation like SUCCEEDED, FAILED or CANCELLED.
        """
        from drivebuildclient.httpUtil import do_get_request
        import dill as pickle
        from time import sleep
        while True:  # Pseudo do-while-loop
            response = do_get_request(self.host, self.port, "/stats/result", {
                "sid": sid.SerializeToString()
            })
            if response.status == 200:
                result = pickle.loads(b"".join(response.readlines()))
                if result == "UNKNOWN":
                    sleep(1)
                else:
                    return result
            else:
                AIExchangeService._print_error(response)
                return "Result could not be determined."

    def get_trace(self, sid: SimulationID, vid: Optional[VehicleID] = None) -> List[Tuple[str, str, int, DataResponse]]:
        """
        Return all the collected data of a single or all participants in a simulation.
        :param sid: The simulation to request all the collected data from.
        :param vid: The vehicle whose collected data has to be returned. If None this method returns all the collected
        data.
        :return: The JSON serialized object representing all the collected data of a simulation or a participant in a
        simulation.
        """
        from drivebuildclient.httpUtil import do_get_request
        import dill as pickle
        args = {
            "sid": sid.SerializeToString()
        }
        if vid:
            args["vid"] = vid.SerializeToString()
        response = do_get_request(self.host, self.port, "/stats/trace", args)
        if response.status == 200:
            response_content = b"".join(response.readlines())
            trace_data = pickle.loads(response_content)
            trace = []
            for entry in trace_data:
                sid = SimulationID()
                sid.sid = str(entry[0])
                vid = VehicleID()
                vid.vid = entry[1]
                data = DataResponse()
                data.ParseFromString(entry[3])
                trace.append((sid, vid, entry[2], data))
            return trace
        else:
            AIExchangeService._print_error(response)
            return "The trace could not be retrieved."

    def get_running_tests(self, user: User) -> SubmissionResult.Submissions:
        """
        Return the currently running tests of the given user.
        :param user: The user to get a list of running simulation for.
        :return: The list of running simulations initiated by the given user.
        """
        from drivebuildclient.httpUtil import do_get_request
        response = do_get_request(self.host, self.port, "/stats/getRunningSids", {
            "user": user.SerializeToString()
        })
        if response.status == 200:
            submission_result = SubmissionResult()
            submission_result.ParseFromString(b"".join(response.readlines()))
            return submission_result.result
        else:
            AIExchangeService._print_error(response)

    def run_tests(self, username: str, password: str, *paths: Path) -> Optional[SubmissionResult.Submissions]:
        """
        Upload the sequence of given files to DriveBuild, execute them and get their associated simulation IDs.
        :param username: The username for login.
        :param password: The password for login.
        :param paths: The sequence of file paths of files or folders containing files to be uploaded.
        :return: A sequence containing simulation IDs for all *valid* test cases uploaded. Returns None iff the upload
        of tests failed or the tests could not be run. Returns an empty list of none of the given test cases was valid.
        """
        from drivebuildclient.httpUtil import do_mixed_request
        from drivebuildclient.aiExchangeMessages_pb2 import User
        from tempfile import NamedTemporaryFile
        from zipfile import ZipFile
        from os import remove, listdir
        from os.path import abspath, basename, isfile, isdir, join, exists
        from drivebuildclient.common import eprint
        temp_file = NamedTemporaryFile(mode="w", suffix=".zip", delete=False)
        temp_file.close()

        with ZipFile(temp_file.name, "w") as write_zip_file:
            def _add_all_files(path: Path) -> None:
                if path.is_file():
                    write_zip_file.write(path.absolute(), path.name)
                elif path.is_dir():
                    for sub_path in path.iterdir():
                        _add_all_files(sub_path)
                elif not path.exists():
                    eprint("Path \"" + str(path) + "\" does not exist.")
                else:
                    eprint("Can not handle path \"" + str(path) + "\".")
            for p in paths:
                _add_all_files(p)
            if len(write_zip_file.filelist) < 2:
                eprint("runTests(...) requires at least two valid files.")
                return None
        user = User()
        user.username = username
        user.password = password
        with open(temp_file.name, "rb") as read_zip_file:
            response = do_mixed_request(self.host, self.port, "/runTests", {
                "user": user.SerializeToString()
            }, b"".join(read_zip_file.readlines()))
        remove(temp_file.name)
        submission_result = SubmissionResult()
        submission_result.ParseFromString(b"".join(response.readlines()))
        if response.status == 200:
            return submission_result.result
        else:
            eprint("Running tests errored:")
            eprint(submission_result.message.message)
            return None



from math import log10, floor
from socket import socket
from typing import List, Callable, Tuple
from threading import Lock


def eprint(*args, **kwargs):
    from sys import stderr
    print(*args, file=stderr, **kwargs)  # TODO Is there some warn(...) equivalent function?


def static_vars(**kwargs):
    """
    Decorator hack for introducing local static variables.
    :param kwargs: The declarations of the static variables like "foo=42".
    :return: The decorated function.
    """

    def decorate(func):
        """
        Decorates the given function with local static variables based on kwargs.
        :param func: The function to decorate.
        :return: The decorated function.
        """
        for k in kwargs:
            setattr(func, k, kwargs[k])
        return func

    return decorate


def create_server(port: int) -> socket:
    from socket import AF_INET, SOCK_STREAM, SOL_SOCKET, SO_REUSEADDR
    server_socket = socket(AF_INET, SOCK_STREAM)
    server_socket.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
    server_socket.bind(("0.0.0.0", port))
    server_socket.listen(1)  # FIXME Determine appropriate value
    return server_socket


def accept_at_server(server_socket: socket, on_accept: Callable[[socket, Tuple[str, int]], None]) -> None:
    while True:
        conn, addr = server_socket.accept()
        print(str(server_socket.getsockname()) + " accepted " + str(addr))
        on_accept(conn, addr)


def create_client(server_host: str, server_port: int) -> socket:
    from socket import AF_INET, SOCK_STREAM
    client_socket = socket(AF_INET, SOCK_STREAM)
    client_socket.connect((server_host, server_port))
    return client_socket


CONTENT_LENGTH_LIMIT: int = 10000000  # 10 millions
# The length of the message transferring the content length
CONTENT_LENGTH_MESSAGE_LENGTH: int = floor(log10(CONTENT_LENGTH_LIMIT)) + 1
MAX_RETRY: int = 100


@static_vars(send_lock=Lock())
def _send_message(sock: socket, message: bytes) -> None:
    message_length = len(message)
    print(str(sock.getpeername()) + " sending " + str(message_length) + " bytes")
    if message_length > CONTENT_LENGTH_LIMIT:
        eprint("Can not send messages longer than " + str(CONTENT_LENGTH_LIMIT) + ".")
    else:
        message_length_message = str(message_length) \
            .ljust(CONTENT_LENGTH_MESSAGE_LENGTH, " ") \
            .encode()
        _send_message.send_lock.acquire()
        sock.sendall(message_length_message)
        sock.sendall(message)
        _send_message.send_lock.release()


# FIXME Introduce method receiving bytes until reaching a certain number
def _recv_message(sock: socket) -> bytes:
    print(str(sock.getsockname()) + " waiting for recv message length")
    tries = 0
    while tries < MAX_RETRY:
        content_length_message = sock.recv(CONTENT_LENGTH_MESSAGE_LENGTH).decode().strip()
        print(str(sock.getsockname()) + " got message length message: " + content_length_message)
        if content_length_message:
            content_length = int(content_length_message)
            print(str(sock.getsockname()) + " waits for receiving a message of length " + str(content_length))
            break
        else:
            tries = tries + 1
    else:
        eprint("Did not receive content length message.")
        return b""
    received_message = b""
    while len(received_message) < content_length:
        received_message = received_message + sock.recv(content_length - len(received_message))
    return received_message


def process_request(sock: socket, handle_message: Callable[[bytes, List[bytes]], bytes]) -> None:
    from drivebuildclient.aiExchangeMessages_pb2 import Num
    action = _recv_message(sock)
    print(str(sock.getsockname()) + " received action " + action.decode())
    num_data = Num()
    num_data.ParseFromString(_recv_message(sock))
    print(str(sock.getsockname()) + " received num_data " + str(num_data.num))
    data = []
    for _ in range(num_data.num):
        data.append(_recv_message(sock))
        print(str(sock.getsockname()) + " received data " + str(data[-1]))
    result = handle_message(action, data)
    print(str(sock.getsockname()) + " sends result")
    _send_message(sock, result)


def process_requests(waiting_socket: socket, handle_message: Callable[[bytes, List[bytes]], bytes]) -> None:
    # FIXME How to recover failures?
    try:
        while True:
            process_request(waiting_socket, handle_message)
    except (ConnectionAbortedError, ConnectionResetError):
        eprint("The socket " + str(waiting_socket.getsockname()) + " was closed.")


def send_request(sock: socket, action: bytes, data: List[bytes]) -> bytes:
    from drivebuildclient.aiExchangeMessages_pb2 import Num
    _send_message(sock, action)
    num_data = Num()
    num_data.num = len(data)
    if num_data.num == 0:
        num_data.num = -1
    _send_message(sock, num_data.SerializeToString())
    for d in data:
        _send_message(sock, d)
    return _recv_message(sock)



import setuptools
import os

with open(os.path.join(os.path.dirname(__file__), "drivebuildclient/README.md"), "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="drivebuild-client",
    version="0.17",
    author="Stefan Huber",
    author_email="huberst@fim.uni-passau.de",
    description="A client for accessing a DriveBuild server.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(
        include=("drivebuildclient",)
    ),
    install_requires=[
        "dill",
        "flask",
        "protobuf"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from typing import Dict, Optional


class DBConnection:
    from typing import Any

    def __init__(self, host: str, port: int, db_name: str, user: str, password: str):
        self._host = host
        self._port = port
        self._db_name = db_name
        self._user = user
        self._password = password

    def run_query(self, query: str, args: Dict[str, Any] = None) -> Optional[Any]:
        from pg8000 import connect
        from drivebuildclient.common import eprint
        import pg8000
        pg8000.paramstyle = "named"
        try:
            with connect(host=self._host, port=self._port, database=self._db_name, user=self._user,
                            password=self._password) as connection:
                connection.autocommit = True
                with connection.cursor() as cursor:
                    return cursor.execute(query, args)
        except Exception as ex:
            eprint("The query \"" + query + "\" failed with " + str(ex))
            return None
